##################### generated by xml-casa (v2) from clearcal.xml ##################
##################### 0e74985bc220c252fb4057cc1a73cb9d ##############################
from __future__ import absolute_import
import numpy
from casatools.typecheck import CasaValidator as _val_ctor
_pc = _val_ctor( )
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from .private.task_clearcal import clearcal as _clearcal_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from casatasks.private.task_logging import except_log as _except_log

class _clearcal:
    """
    clearcal ---- Re-initializes the calibration for a visibility data set

    
    Clearcal reinitializes the calibration columns in a measurement
    set. Specificially, it will set the MODEL_DATA column (if present) to
    unity in total intensity and zero in polarization, and it will set the
    CORRECTED_DATA column to the original (observed) DATA in the DATA
    column.  Use the field and spw parameters to select which data to
    initialize.  If the dataset does not yet have the scratch columns,
    they will be created (MODEL_DATA only if addmodel=True) and initilized
    for the whole dataset (field, spw, and intent will be ignored in this
    case).

    --------- parameter descriptions ---------------------------------------------

    vis      Name of input visibility file (MS)
             Default: none
             
                Example: vis='ngc5921.ms'
    field    Select field using field id(s) or field name(s)
             default: '' (all fields)
             
             Use 'go listobs' to obtain the list id's or
             names. If field string is a non-negative integer,
             it is assumed a field index,  otherwise, it is
             assumed a field name.
             
                Examples:
                field='0~2'; field ids 0,1,2
                field='0,4,5~7'; field ids 0,4,5,6,7
                field='3C286,3C295'; field named 3C286 and
                3C295
                field = '3,4C*'; field id 3, all names
                starting with 4C
    spw      Select spectral window/channels
             
             Examples:
             spw='0~2,4'; spectral windows 0,1,2,4 (all
             channels)
             spw='<2';  spectral windows less than 2
             (i.e. 0,1)
             spw='0:5~61'; spw 0, channels 5 to 61,
             INCLUSIVE
             spw='*:5~61'; all spw with channels 5 to 61
             spw='0,10,3:3~45'; spw 0,10 all channels, spw
             3, channels 3 to 45.
             spw='0~2:2~6'; spw 0,1,2 with channels 2
             through 6 in each.
             spw='0:0~10;15~60'; spectral window 0 with
             channels 0-10,15-60. (NOTE ';' to separate
             channel selections)
             spw='0:0~10^2,1:20~30^5'; spw 0, channels
             0,2,4,6,8,10, spw 1, channels 20,25,30 
             type 'help par.selection' for more examples.
    intent   Select observing intent
             default: '' (no selection by intent)
             
                Example: intent='*BANDPASS*'  (selects data
                labelled with BANDPASS intent)
    addmodel add MODEL_DATA along with CORRECTED_DATA?
             Default: False (model will not be added)
             Options: False|True
             
             If False, it will add/reset only CORRECTED_DATA,
             model visibilities will then be evaluated when
             needed.
    [1;42mRETURNS[1;m     void

    --------- examples -----------------------------------------------------------

    
    
    For more information, see the task pages of clearcal in CASA Docs:
    
    https://casa.nrao.edu/casadocs/
    
    


    """

    _info_group_ = """calibration"""
    _info_desc_ = """Re-initializes the calibration for a visibility data set"""

    def __call__( self, vis='', field='', spw='', intent='', addmodel=False ):
        schema = {'vis': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'field': {'type': 'cStr', 'coerce': _coerce.to_str}, 'spw': {'type': 'cStr', 'coerce': _coerce.to_str}, 'intent': {'type': 'cStr', 'coerce': _coerce.to_str}, 'addmodel': {'type': 'cBool'}}
        doc = {'vis': vis, 'field': field, 'spw': spw, 'intent': intent, 'addmodel': addmodel}
        assert _pc.validate(doc,schema), create_error_string(_pc.errors)
        _logging_state_ = _start_log( 'clearcal', [ 'vis=' + repr(_pc.document['vis']), 'field=' + repr(_pc.document['field']), 'spw=' + repr(_pc.document['spw']), 'intent=' + repr(_pc.document['intent']), 'addmodel=' + repr(_pc.document['addmodel']) ] )
        task_result = None
        try:
            task_result = _clearcal_t( _pc.document['vis'], _pc.document['field'], _pc.document['spw'], _pc.document['intent'], _pc.document['addmodel'] )
        except Exception as exc:
            _except_log('clearcal', exc)
            raise
        finally:
            task_result = _end_log( _logging_state_, 'clearcal', task_result )
        return task_result

clearcal = _clearcal( )

